/* $Header: /home/powell/mod10/TERRA/v5.0.0/RCS/MOD_PR10.h,v 1.3 2005/01/13 16:43:10 powell Exp $

***************************************************************************************
!C-INC

!Description:
	This is the header file which contains the definitions of variables and
	structures that are used by the program.

Program returns:
	N/A

!Input Parameters:
	N/A

!Output Parameters:
	N/A

!Revision History:

   $Log: MOD_PR10.h,v $
   Revision 1.2  2005/05/19 18:20:05  powell
   Fix phone number.

   Revision 1.1  2005/05/19 17:42:03  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 971.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 974.1
        SAIC
        4600 Powder Mill Road Beltsville, MD 20705
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 971.0
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:

!Design Notes:
	This is the Collection 5 version of the code.

!END
***************************************************************************************/

#ifdef THIS_IS_MAIN_PROGRAM
char	*rcs2="$Id: MOD_PR10.h,v 1.2 2005/05/19 18:20:05 powell Exp $";
#endif

/*** Defines for .pcf files */
#define L1B_INPUT		700001
#define L1B1KM_INPUT		700002
#define CLOUD_INPUT		422500
#define GEO_INPUT		600000
#define SNOW_OUTPUT		210100
#define SNOW_5KM_OUTPUT		210105
#define MCF_FILE		210250
#define MCF_FILE_5KM		210253

#define ALGORITHM_PACKAGE_ACCEPTANCE_DATE	210260
#define ALGORITHM_PACKAGE_MATURITY_CODE		210261
#define ALGORITHM_PACKAGE_VERSION		210262
#define REPROCESSING_PLANNED			800503
#define REPROCESSING_ACTUAL			800504
#define SCIENCE_QUALITY_FLAG_EXPLANATION	800506
#define PROCESSING_ENVIRONMENT			800550
#define PGE_VERSION				800500

#define LOCAL_VERSION_ID	"SCF V5.0.1"
#define CORE			"CoreMetadata.0"
#define ARCHIVE			"ArchiveMetadata.0"

/*** Boolean test bits for the input cloud mask SDS. ***/
#define CLOUD_MASK_CF		( 1<<0 )
#define CLOUD_MASK_1_CF		( 1<<1 )
#define CLOUD_MASK_2_CF		( 1<<2 )
#define CLOUD_MASK_3_CF		( 1<<3 )
#define CLOUD_MASK_14_CF	( 1<<4 )
#define CLOUD_MASK_19_CF	( 1<<5 )
#define CLOUD_MASK_20_CF	( 1<<6 )

/*** Possible values for the output snow_cover SDS. ***/
#define FILL_VALUE			255
#define DETECTOR_SATURATED		254
#define SNOW_FOREST			200	/*** This was originally 211 ***/
#define SNOW				200
#define LAKE_ICE			100
#define CLOUD_OBSCURED			50
#define OCEAN				39
#define LAKE				37
#define NO_SNOW				25
#define NIGHT				11
/****  #define SCAN_ANGLE_EXCEEDED	3	***/
#define NO_DECISION			1
#define L1B_MISSING_DATA		0

/*** Possible values for the output Snow Cover PixelQA SDS. ***/

#define QA_OTHER			1
#define QA_GOOD				0
#define QA_ANTARICTA			251	/*  Not Used  */
#define QA_LAND				253	/*  Not Used  */
#define QA_OCEAN			254
#define QA_FILL				255	/*  Not Used  */

#define MASK_02		( 1<<1 | 1<<0 )
#define MASK_01		~MASK_02

#define XTRUEX		1
#define XFALSEX		0
#define OK		PGS_S_SUCCESS

/*** The value of of MODIS channel 6 uncertainty SDS for "2nd sample". ***/
#define UNCERTAINTY_VALUE	14

/*** Possible L1B unusable values. ***/
#define L1B_MISSING		65534
#define SATURATED_DETECTOR	65533
#define UNUSABLE_L1B		65500
#define MAX_USABLE_L1B		32767
#define L1B_FILL		65535

/*** Possible values for the input Land/SeaMask SDS in the GEO file. ***/
/*** From Tom Logan's 6/23/97 email to Robert Wolfe. ***/
#define SHALLOW_OCEAN		0
#define LAND			1
#define COASTLINES		2
#define SHALLOW_INLAND_WATER	3
#define EPHEMERAL_WATER		4
#define DEEP_INLAND_WATER	5
#define CONTINENTAL_SHELF_OCEAN	6
#define DEEP_OCEAN		7

#define MAX_SFC_TEMP_FOR_SNOW   283.0

enum AutoQualFlag		{PASSED, SUSPECT};
enum OpType			{OPEN,READ,WRITE,CLOSE,TERMINATE};

#define MAX_SAMPS		2708
#define MAX_5KM_SAMPS		271
#define MAX_LINES		4080
#define MAX_5KM_LINES		408
#define MAX_GRING_POINTS	4

#define ADDITIONAL_ATTRIBUTES	4

struct COUNTS {
  long	FirstEntryInStruct;	/*++++ THIS MUST BE THE FIRST ENTRY IN THE STRUCT ++++*/

  long	AllPixelsProcessed;
  long  DeadDetectorPixels;
  long	InvalidInput;
  long  LandDayPixels;
  long	LandPixels;
  long	NightPixels;
  long	NoDecision;
  long  QA_Good;
  long  QA_MissingData;
  long  QA_Ocean;
  long  QA_Other;
  long  Snow_CloudObscured;
  long  Snow_DetectorSaturated;
  long  Snow_FillValue;
  long  Snow_L1BMissingData;
  long  Snow_Lake;
  long  Snow_LakeIce;
  long  Snow_Night;
  long  Snow_NoDecision;
  long  Snow_NoSnow;
  long  Snow_Ocean;
  long  Snow_Rejected_Warm;
  long  Snow_Snow;
  };

struct COINCIDENCE {
  float64	GRingPointLatitude[MAX_GRING_POINTS];
  float64	GRingPointLongitude[MAX_GRING_POINTS];
  char		RangeBeginningDate[64], RangeBeginningTime[64];
};

struct CHECK {
  struct COINCIDENCE l1b;
  struct COINCIDENCE geo;
  struct COINCIDENCE cld;
};

struct Meta5km {
  float64       EquatorCrossingLongitude;
  float64       GRingPointLongitude[MAX_GRING_POINTS];
  float64	GRingPointLatitude[MAX_GRING_POINTS];
  float64       EastBoundingCoordinate, WestBoundingCoordinate;
  float64       NorthBoundingCoordinate, SouthBoundingCoordinate;

  int32 GRingPointSequenceNo[MAX_GRING_POINTS];
  int32 OrbitNumber;
  int32 QAPercentCloudCover;
  int32 QAPercentMissingData;

  char	*p_InputPointer[4];
  char  *p_LocalInputGranuleID[4];
  char  *AdditionalAttributeName[ADDITIONAL_ATTRIBUTES];

  char  AdditionalAttributeParm[ADDITIONAL_ATTRIBUTES][42];
  char  AlgorithmPackageAcceptanceDate[24];
  char  AlgorithmPackageMaturityCode[40];
  char	AlgorithmPackageName[128];
  char  AlgorithmPackageVersion[24];
  char  AncillaryInputPointer[128];
  char  AncillaryInputType[32];
  char  AutomaticQualityFlagExplanation[512];
  char  AutomaticQualityFlag[16];
  char  DayNightFlag[16];
  char  EquatorCrossingDate[12];
  char  EquatorCrossingTime[24];
  char  ExclusionGRingFlag[8];
  char  LocalGranuleID[128];
  char  LocalVersionID[16];
  char  LongName[64];
  char  PGEversion[12];
  char  ParameterName[64];
  char  ParameterValue[8][16];
  char  ProcessingCenter[16];
  char  ProcessingEnvironment[256];
  char  ProcessingDateTime[32];
  char  RangeBeginningDate[12];
  char  RangeBeginningTime[24];
  char  RangeEndingDate[12];
  char  RangeEndingTime[24];
  char  ReprocessingActual[256];
  char  ReprocessingPlanned[256];
  char	ScienceQualityFlagExplanation[512];
  char  SPSOParameters[16];
};
