###########################################################################
# !make
#
# !Makefile Name: MOD_PR10.mk
#
# !Description:   Makefile for MOD_PR10 Version 2 L2 snow code.
#		  This makefile used to generate MOD_PR10.exe
#
# !Variables:
#
# 	VARIABLES     DESCRIPTION
# 	~~~~~~~~~     ~~~~~~~~~~~
# 	TARGET        Program executable name
# 	ADD_CFLAGS    Additional compiler options
# 	LIB           Libraries
# 	MATH          Math Libraries
# 	SRCS          Source files
# 	OBJ           Object files
# 	INC           Include files
# 	INC_FILES     Additional include files
# 	SYSTEM        System
#
# !Env Variables
#
# 	ENV VARIABLES DESCRIPTION
# 	~~~~~~~~~~~~~ ~~~~~~~~~~~
# 	CFLAGS        Compiler flags which are set by the script
#		      (e.g. v2_n32_f77)
# 	CC            The compiler set by the script
# 	API_INC       Include directory of MODIS Appl Prog Interface set
#		      by the script
# 	PGSINC        Include directory of PGS toolkit set by the script
# 	HDFINC        Include directory of HDF set by the script
# 	HDFEOS_INC    Include directory of HDFEOS set by the script
# 	API_LIB       Library of MODIS API set the the script
# 	PGSLIB        Library of PGS toolkit set by the script
# 	HDFLIB        Library of HDF set by the script
# 	HDFEOS_LIB    Library of HDFEOS set by the script
#
# !Team-Unique Header:
#    	This software was developed by MODIS Science Data Support Team
#    	for the National Aeronautics and Space Administration,
#    	Goddard Space Flight Center, under contract NAS5-32373.
#
# !Revision History:
# 	$Log: MOD_PR10.mk,v $
# Revision 1.4  2000/08/30  19:05:59  powell
# Changes for ver 2.4.1.0.
#
# Revision 1.3  1999/12/21  18:03:47  powell
# Version 2.4.0 updates
#
# Revision 1.2  1998/12/30  15:24:11  powell
# Version 2.2 delivery
#
#
# !Note:
#       1) must follow MAPI, SDP TK, HDF, and other header/libs order
#       2) HDFLIB must be explicit and in order of -lmfhdf, -ldf, -ljpeg, -lz.
#       3) system environments, such as APIINC, PGSINC, HDFINC, PGSLIB,
#          API_LIB etc. should be defined befor execute this makefile.
#
# !END
##############################################################################

# Compiler options are set based on the environment.
target:
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
            $(MAKE) $(TARGET) -f MOD_PR10.mk \
                    ADD_CFLAGS="-O3 -DIRIS4 -r10000 " \
            ;; \
          linux ) \
            $(MAKE) $(TARGET) -f MOD_PR10.mk \
                    ADD_CFLAGS=" -O3 -fwritable-strings" \
            ;; \
          * ) \
            echo " ERROR: Enviornment variable BRAND (set to  \"$(BRAND)\") not recognized." >&2 ; \
            exit 1 ;; \
        esac

# Define executable name
TARGET = MOD_PR10.exe

# Combine the includes using pre-defined includes and your includes
INC_HDF5OPT = -I$(HDF5INC)
INC =   -I$(PGSINC) -I$(HDFINC) -I$(HDFEOS_INC) $(INC_HDF5OPT:-I=)

# Combine the libraries using pre-defined libraries and your library
LIB =   -L$(HDFEOS_LIB) -lhdfeos -lGctp \
        -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz \
        -L$(PGSLIB) -lPGSTK \
        -L$(HDF5LIB) -lhdf5 \
        -lm


# Define object files
OBJ =	MOD_PR10_AAmain.o			\
	MOD_PR10_Compute_Snow.o			\
	MOD_PR10_Compute_FractionalSnow.o 	\
	MOD_PR10_CopyL1BmetaToSnow.o		\
	MOD_PR10_CopyGEOmetaToSnow.o		\
	MOD_PR10_MakeMeta.o			\
	MOD_PR10_Process_Cloud.o		\
	MOD_PR10_Process_GEO.o			\
	MOD_PR10_Process_L1B.o			\
	MOD_PR10_Process_SnowFile.o		\
	MOD_PR10_Process_5kmSnowFile.o		\
	MOD_PR10_UpdateCounts.o         	\
	MOD_PR10_Compute_SfcTemp.o

# Additional include files
INC_FILES = MOD_PR10.h MOD_PR10_ActionMessages.h
# Define additional header files
ADDHDRS = $(PGSINC)/PGS_PC.h

# Make the process
$(TARGET) :	$(OBJ)
	$(CC)	$(CFLAGS) $(ADD_CFLAGS) $(OBJ) $(LIB) -o $(TARGET)

.c.o :	$(INC_FILES) $(ADDHDRS)
	$(CC)	$(CFLAGS) $(ADD_CFLAGS) $(INC) -c  $< -o $@

# Delete object files
clean:
		/bin/rm $(OBJ)

# Delete the executable file $TARGET
clean_exe:
		/bin/rm $(TARGET)
#******************** End of make file ********************
