/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_CopyGEOmetaToSnow.c,v 1.2 2005/05/19 18:14:18 powell Exp $

***************************************************************************************
!C

!Description:
	This function copies all relevant global meta data from the GEO input file to
	the snow output file.  It is called once by the program.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	GEO_INPUT		The PCF number for the GEO input file name
	Meta500Handles		Metadata group handle in MCF

!Output Parameters:
	None

!Revision History:

   $Log: MOD_PR10_CopyGEOmetaToSnow.c,v $
   Revision 1.2  2005/05/19 18:14:18  powell
   Fix phone number.

   Revision 1.1  2005/05/19 17:30:23  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05   Revision 1.1  2005/05/19 17:13:10  powell

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:
	
!Design Notes:
	This is the Collection 5 version of the code. The global metadata are copied
	from the GEO input by use of the PGS_MET_GetPCAttr and PGS_MET_SetAttr routines.
	The names of the metadata items are hard coded in the program.

!END
***************************************************************************************/

#include <stdio.h>
#include <string.h>
#include "hdfi.h"
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

/*** Global Variables ***/
char		GranuleNumber[64];
extern char	HaltMsg[];
extern PGSt_MET_all_handles		  Meta500Handles;
extern char	RCSidentString[];
extern struct Meta5km km;

/*====================================================================================*/

void CopyGEOmetaToSnow( )
{
  char *rcs="$Id: MOD_PR10_CopyGEOmetaToSnow.c,v 1.2 2005/05/19 18:14:18 powell Exp $";

  /*** Copy character string metadata from GEO input to output. ***/
  static char	FirstTimeFlag = XTRUEX;

  PGSt_integer	ver;
  char		CharStg[256] = "\0";
  void		*local = CharStg;


  /*** Add the RCS identification to the global string. ***/
  if( FirstTimeFlag == XTRUEX ) {
    FirstTimeFlag = XFALSEX;
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );
  }


  /*** Process DOUBLE meta data. ***/
    ver = 1;
    if(PGS_MET_GetPCAttr(GEO_INPUT, ver, ARCHIVE,
			 "EastBoundingCoordinate", &km.EastBoundingCoordinate ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting EastBoundingCoordinate");
    if(PGS_MET_SetAttr(Meta500Handles[2],
		       "EastBoundingCoordinate", &km.EastBoundingCoordinate) != OK )
      HALT2(ACT05,"FAILURE putting EastBoundingCoordinate");

    ver = 1;
    if(PGS_MET_GetPCAttr(GEO_INPUT, ver, ARCHIVE,
			 "WestBoundingCoordinate", &km.WestBoundingCoordinate ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting WestBoundingCoordinate");
    if(PGS_MET_SetAttr(Meta500Handles[2],
		       "WestBoundingCoordinate", &km.WestBoundingCoordinate) != OK )
      HALT2(ACT05,"FAILURE putting WestBoundingCoordinate");

    ver = 1;
    if(PGS_MET_GetPCAttr(GEO_INPUT, ver, ARCHIVE,
			 "NorthBoundingCoordinate", &km.NorthBoundingCoordinate ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting NorthBoundingCoordinate");
    if(PGS_MET_SetAttr(Meta500Handles[2],
		       "NorthBoundingCoordinate", &km.NorthBoundingCoordinate) != OK )
      HALT2(ACT05,"FAILURE putting NorthBoundingCoordinate");

    ver = 1;
    if(PGS_MET_GetPCAttr(GEO_INPUT, ver, ARCHIVE,
			 "SouthBoundingCoordinate", &km.SouthBoundingCoordinate ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting SouthBoundingCoordinate");
    if(PGS_MET_SetAttr(Meta500Handles[2],
		       "SouthBoundingCoordinate", &km.SouthBoundingCoordinate) != OK )
      HALT2(ACT05,"FAILURE putting SouthBoundingCoordinate");



  /*** Get the GranuleNumber from the GEO file and put it in a global variable
    for pick up by MakeMeta.c. ***/

  ver = 1;
  if(PGS_MET_GetPCAttr(GEO_INPUT,ver,CORE,"AdditionalAttributeName.1",&local) != OK )
    HALT2(ACT07,"PGS_MET_GetPCAttr failure getting AdditionalAttributeName for GranuleNumber value from GEO file");


  if( strcmp( CharStg, "GRANULENUMBER" ) != 0 )
    HALT2(ACT07,"AdditionalAttributeName for GranuleNumber from GEO file incorrect.");

  ver = 1;
  local = GranuleNumber;
  GranuleNumber[0] = '\0';
  if(PGS_MET_GetPCAttr(GEO_INPUT, ver, CORE, "ParameterValue.1",  &local ) != OK )
    HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GranuleNumber value from GEO file");
}
