/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_CopyL1BmetaToSnow.c,v 1.2 2005/05/19 18:15:48 powell Exp $

***************************************************************************************
!C

!Description:
	This function copies all relevant global meta data from the L1B input file to
	the snow output file.  It is called once by the program.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	L1B_INPUT	The PCF number for the L1B input file name
	Meta500Handles	Metadata group handle in MCF

!Output Parameters:
	None

!Revision History:

   $Log: MOD_PR10_CopyL1BmetaToSnow.c,v $
   Revision 1.2  2005/05/19 18:15:48  powell
   Fix phone number.

   Revision 1.1  2005/05/19 17:30:23  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:

!Design Notes:
	This is the Collection 5 version of the code. The global meta data are copied
	from the L1B input by use of the PGS_MET_GetPCAttr and PGS_MET_SetAttr routines.
	The names of the meta data items are hard coded in the program.

!END
***************************************************************************************/

#include <stdio.h>
#include <string.h>
#include  <time.h>

#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "PGS_PC.h"
#include "hdf.h"
#include "mfhdf.h"
#include "HdfEosDef.h"

#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

#define TIMECODEASIZE	28
#define TERRA_ESDT	"MOD10_L2.A"
#define TERRA_ESDT_5KM	"MOD10L2C.A"

/*** Global Variables ***/
extern char			HaltMsg[];
extern struct tm		CurrentTime;
extern PGSt_MET_all_handles	Meta500Handles;
extern int32			SnowHDFhandle;
extern char			RCSidentString[];
extern struct Meta5km		km;

/*====================================================================================*/

void CopyL1BmetaToSnow( )
{
  char *rcs="$Id: MOD_PR10_CopyL1BmetaToSnow.c,v 1.2 2005/05/19 18:15:48 powell Exp $";

  /*** Copy character string metadata from L1B input to output. ***/
  PGSt_integer	j, version;
  static char	FirstTimeFlag = XTRUEX;
  char		CalibrationQuality[128], MissionPhase[128], NadirPointing[32];
  char		VersionID[32];
  char		timea[128],  timeb[128], AttrStg[256];
  void		*local;

  char	*CharStg[] = {
    "RangeBeginningDate",	/* This must be the first  of CharStg. */
    "RangeBeginningTime",	/* This must be the second of CharStg. */
    "RangeEndingDate",
    "RangeEndingTime",
    "ExclusionGRingFlag.1",
    "EquatorCrossingDate.1",
    "EquatorCrossingTime.1" };

  char	*CharStgPntr[] = {	/* These must be in same order as CharStg. */
    km.RangeBeginningDate,
    km.RangeBeginningTime,
    km.RangeEndingDate,
    km.RangeEndingTime,
    km.ExclusionGRingFlag,
    km.EquatorCrossingDate,
    km.EquatorCrossingTime };

  char	*DoubleStg[] = {
    "GRingPointLatitude.1",
    "GRingPointLongitude.1",
    "EquatorCrossingLongitude.1" };

  float64	*DoubleStgPntr[] = {	/* These must be in same order as DoubleStg. */
    km.GRingPointLatitude,
    km.GRingPointLongitude,
    &km.EquatorCrossingLongitude };

  char	ChrVal[16][PGSd_PC_VALUE_LENGTH_MAX];



  /*** Add the RCS identification to the global string. ***/
  if( FirstTimeFlag == XTRUEX ) {
    FirstTimeFlag = XFALSEX;
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );
  }


  /*** Process STRING meta data. ***/
  for(j=0; j<sizeof(CharStg)/sizeof(*CharStg); j++ ) {

    version = 1;
    ChrVal[j][0] = '\0';
    local = ChrVal[j];

    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, CharStg[j], &local ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting a CORE string value");

    if(PGS_MET_SetAttr(Meta500Handles[1], CharStg[j], &local ) != OK )
      HALT2(ACT05,"FAILURE putting an L1B input CORE string value");

    strcpy( CharStgPntr[j], ChrVal[j] );
  }


  /*** Process INTEGER GRingPointSequenceNo meta data. ***/
  {
    int32	jj, i[16]={-999,-999,-999,-999,-999,-999,-999,-999,-999,-999,-999,-999,-999,-999,-999,-999};
    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "GRingPointSequenceNo.1", &i) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointSequenceNo");

    if(PGS_MET_SetAttr(Meta500Handles[1], "GRingPointSequenceNo.1", &i ) != OK )
      HALT2(ACT05,"FAILURE putting an L1B input GRingPointSequenceNo");

    /*** Check that the number of Gring points is compatible with the compiled max. ***/
    for(jj=16; jj>0; jj--)
      if(i[jj-1] > -999) break;
    if( jj != MAX_GRING_POINTS )
      HALT2(ACT05,"No. of GringPoints != compiled value MAX_GRING_POINTS.");

    for(jj=0; jj<MAX_GRING_POINTS; jj++ )
      km.GRingPointSequenceNo[jj] = i[jj];
  }


  /*** Process INTEGER OrbitNumber meta data. ***/
  {
    int32	jj, i[16]={-999,-999,-999,-999,-999,-999,-999,-999,
			   -999,-999,-999,-999,-999,-999,-999,-999};
    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "OrbitNumber.1", &i) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting OrbitNumber");

    if(PGS_MET_SetAttr(Meta500Handles[1], "OrbitNumber.1", &i ) != OK )
      HALT2(ACT05,"FAILURE putting an L1B input OrbitNumber");

    for(jj=16; jj>0; jj--)
      if(i[jj-1] > -999) break;
    if( jj != 1 )
      HALT2(ACT05,"Only one OrbitNumber allowed in the compiled code.");

    km.OrbitNumber = i[0];
  }


  /*** Process DOUBLE meta data. ***/
  for(j=0; j<sizeof(DoubleStg)/sizeof(*DoubleStg); j++ ) {
    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT,version, CORE, DoubleStg[j], DoubleStgPntr[j]) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting a CORE double value");

    if(PGS_MET_SetAttr(Meta500Handles[1], DoubleStg[j], DoubleStgPntr[j] ) != OK )
      HALT2(ACT05,"FAILURE putting an L1B input CORE double value");
  }

  /*** Retrieve the VersionID from MCF file for use in LocalGranuleID.***/
  if(PGS_MET_GetSetAttr(Meta500Handles[1], "VersionID", &local) != OK )
      HALT2(ACT07,"PGS_MET_GetSetAttr failure retrieving VersionID value from MCF file");
  sprintf(VersionID,"%03d",local);


  /*** Make the LocalGranuleID from RangeBeginningDateTime & production DateTime. ***/
  {
    char procDateTimeA[30], procDateTimeB[30];
    struct tm *curtime	 = NULL;
    time_t t;
    char	TimeNow[64] = {NULL};

    memset( procDateTimeA, 0, sizeof(procDateTimeA) );
    memset( procDateTimeB, 0, sizeof(procDateTimeB) );

    /*** Generate processing time (procDateTimeB). ***/
    if((t = time(NULL)) == -1)
      HALT2(ACT08,"time() failed ");

    if(!(curtime = (struct tm *)localtime(&t)))
      HALT2(ACT08,"localtime() failed");

    /*** Obtain CCSDS ASCII time code A. ***/
    if((strftime(procDateTimeA,TIMECODEASIZE,"%Y-%m-%dT%H:%M:%S.000000Z",curtime)) == 0)
      HALT2(ACT08,"strftime() failed");

    if(PGS_TD_ASCIItime_AtoB(procDateTimeA, procDateTimeB) != OK )
      HALT2(ACT08,"PGS_TD_ASCIItime_AtoB() failed");

    /*** The Information of current time in ASCIItimeB type was installed
      in procDateTimeB string and ready for use. ***/

    sprintf( timea, "%sT%sZ", ChrVal[0], ChrVal[1] );

    if(PGS_TD_ASCIItime_AtoB( timea, timeb ) != OK )
      HALT2(ACT08,"PGS_TD_ASCIItime_AtoB failed");

    timeb[4] = timeb[8] = timeb[11] = timeb[14] = '\0';

    /*** Get the current GM time of this run. ***/
    if( (strftime(TimeNow, sizeof(TimeNow), "%Y%j%H%M%S", &CurrentTime) ) == 0 )
      HALT2(ACT08,"strftime failed while converting the GM time for LocalGranuleID.");

    sprintf( km.LocalGranuleID, "%s%s%s.%s%s.%s.%s.hdf",TERRA_ESDT,
	     timeb, &timeb[5], &timeb[9], &timeb[12], VersionID, TimeNow );

    local = km.LocalGranuleID;
    if(PGS_MET_SetAttr(Meta500Handles[1],"LocalGranuleID",&local ) != OK )
      HALT2(ACT05,"FAILURE putting LocalGranuleID in CORE");

    sprintf( km.LocalGranuleID, "%s%s%s.%s%s.%s.%s.hdf",  TERRA_ESDT_5KM,
	     timeb, &timeb[5], &timeb[9], &timeb[12], VersionID, TimeNow );
  }



  /*** Get the global attribute L1BCalibrationQuality. ***/

  version = 1;
  local = AttrStg;
  AttrStg[0] = '\0';
  if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE,"AdditionalAttributeName.7", &local)!=OK)
    HALT2(ACT01,"PGS_MET_GetPCAttr failed: AdditionalAttributeName.7");

  if( strcmp( AttrStg, "CalibrationQuality" ) != 0 )
    HALT2(ACT02,"AdditionalAttributeName.7 != CalibrationQuality.");

  version = 1;
  local = CalibrationQuality;
  CalibrationQuality[0] = '\0';
  if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "ParameterValue.7", &local ) != OK )
    HALT2(ACT03,"PGS_MET_GetPCAttr failed: Parameter.7 (CalibrationQuality)");

  if( SDsetattr( SnowHDFhandle,"L1BCalibrationQuality", DFNT_CHAR,
		 strlen(CalibrationQuality), CalibrationQuality ) != OK )
    HALT2(ACT02,"Can't SDsetattr for the L1BCalibrationQuality");


  /*** Get the global attribute L1BMissionPhase. ***/

  version = 1;
  local = AttrStg;
  AttrStg[0] = '\0';
  if(PGS_MET_GetPCAttr(L1B_INPUT,version,CORE,"AdditionalAttributeName.8",&local) != OK )
    HALT2(ACT01,"PGS_MET_GetPCAttr failed: AdditionalAttributeName.8 (MissionPhase)");

  if( strcmp( AttrStg, "MissionPhase" ) != 0 )
    HALT2(ACT02,"AdditionalAttributeName.8 != MissionPhase.");

  version = 1;
  local = MissionPhase;
  MissionPhase[0] = '\0';
  if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "ParameterValue.8", &local ) != OK )
    HALT2(ACT03,"PGS_MET_GetPCAttr failed: Parameter.8 (MissionPhase)");

  if( SDsetattr( SnowHDFhandle,"L1BMissionPhase", DFNT_CHAR,
		 strlen(MissionPhase), MissionPhase ) != OK )
    HALT2(ACT02,"Can't SDsetattr for the L1BMissionPhase");


  version = 1;
  local = AttrStg;
  AttrStg[0] = '\0';
  if(PGS_MET_GetPCAttr(L1B_INPUT,version,CORE,"AdditionalAttributeName.9",&local) != OK )
    HALT2(ACT01,"PGS_MET_GetPCAttr failed: AdditionalAttributeName.9 (NadirPointing)");

  if( strcmp( AttrStg, "NadirPointing" ) != 0 )
    HALT2(ACT02,"AdditionalAttributeName.9 != NadirPointing.");

  version = 1;
  local = NadirPointing;
  NadirPointing[0] = '\0';
  if(PGS_MET_GetPCAttr(L1B_INPUT,version,CORE,"ParameterValue.9", &local ) != OK )
    HALT2(ACT03,"PGS_MET_GetPCAttr failed: Parameter.9 (NadirPointing)");

  if( SDsetattr( SnowHDFhandle,"L1BNadirPointing", DFNT_CHAR,
		 strlen(NadirPointing), NadirPointing ) != OK )
    HALT2(ACT02,"Can't SDsetattr for the L1BNadirPointing");


  /*** Make snow file global attributes from L1B data. ***/
  version = 1;
  ChrVal[j][0] = '\0';
  local = ChrVal[0];
  if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "VersionID", &local ) != OK )
    HALT2(ACT07,"PGS_MET_GetPCAttr failure getting a CORE VersionID");

  if( SDsetattr(SnowHDFhandle,"L1BVersionID",DFNT_CHAR,strlen(ChrVal[0]),ChrVal[0])!=OK)
    HALT2(ACT02,"Can't SDsetattr for the L1BVersionID");
}
