/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_MakeMeta.c,v 1.2 2005/05/19 18:16:34 powell Exp $

****************************************************************************************
!C

!Description:
	This function produces all relevant global metadata, other than that copied
	from the L1B input file by CopyL1BmetaToSnow.  It is called once by the program.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	Meta500Handles		Metadata group handle in MCF

!Output Parameters:
	None

!Revision History:
  $Log: MOD_PR10_MakeMeta.c,v $
  Revision 1.2  2005/05/19 18:16:34  powell
  Fix phone number.

  Revision 1.1  2005/05/19 17:30:23  powell
  Initial revision



!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:


!Design Notes:
	This is the Collection 5 version of the code. The global meta data are made by
	use of the PGS_MET_SetAttr routine.  The names of the metadata items are
	hardcoded in the program.  Some meta data values are read from the PCF.

!END
***************************************************************************************/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include "hdfi.h"
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

/*** Global Variables ***/
extern struct Meta5km		km;
extern struct COUNTS		cnt;
extern enum AutoQualFlag	AutomaticQualityFlag;
extern PGSt_MET_all_handles	Meta500Handles;
extern float32			pct_nominal, pct_abnormal;
extern char			in_FileName[], in1km_FileName[];
extern char			geo_FileName[], cloud_FileName[];
extern char			HaltMsg[], GranuleNumber[], RCSidentString[];

/*====================================================================================*/

void MakeMeta( )
{
  char *rcs="$Id: MOD_PR10_MakeMeta.c,v 1.2 2005/05/19 18:16:34 powell Exp $";

  static char	FirstTimeFlag = XTRUEX;
  char		CharString[64];
  void		*local,	 *loc2 = CharString, *loc3 = GranuleNumber;
  static char	URLin[PGSd_PC_UREF_LENGTH_MAX];
  static char	URL1KMin[PGSd_PC_UREF_LENGTH_MAX];
  static char	URLcloud[PGSd_PC_UREF_LENGTH_MAX];
  static char	URLgeo[PGSd_PC_UREF_LENGTH_MAX];
  static char	FILE_in[PGSd_PC_UREF_LENGTH_MAX];
  static char	FILE_in1km[PGSd_PC_UREF_LENGTH_MAX];
  static char	FILE_cloud[PGSd_PC_UREF_LENGTH_MAX];
  static char	FILE_geo[PGSd_PC_UREF_LENGTH_MAX];
  PGSt_integer	version, *version_p = &version;
  char		*n[] = { URLin, URL1KMin, URLcloud, URLgeo };
  char		*f[] = { FILE_in, FILE_in1km, FILE_cloud, FILE_geo };
  void		*q;
  long		DenomLandDay;
  float32	FloatVal;



  /*** Prevent divide checks in the code below. ***/
  DenomLandDay = cnt.LandDayPixels > 0    ?  cnt.LandDayPixels    :  1;


  /*** Add the RCS identification to the global string. ***/
  if( FirstTimeFlag == XTRUEX ) {
    FirstTimeFlag = XFALSEX;
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );
  }

  /*** Retrieve parameters from PCF file.  ***/
  if(PGS_PC_GetConfigData( REPROCESSING_ACTUAL, km.ReprocessingActual ) != OK)
    HALT2(ACT04,"Error getting ReprocessingActual from PCF");
  q = km.ReprocessingActual;
  if(PGS_MET_SetAttr(Meta500Handles[1],"ReprocessingActual",&q ) != OK )
    HALT2(ACT05,"FAILURE putting ReprocessingActual");

  if(PGS_PC_GetConfigData( REPROCESSING_PLANNED, km.ReprocessingPlanned ) != OK)
    HALT2(ACT04,"Error getting ReprocessingPlanned from PCF");
  q = km.ReprocessingPlanned;
  if(PGS_MET_SetAttr(Meta500Handles[1],"ReprocessingPlanned",&q ) != OK )
    HALT2(ACT05,"FAILURE putting ReprocessingPlanned");

  if(PGS_PC_GetConfigData( SCIENCE_QUALITY_FLAG_EXPLANATION,
			   km.ScienceQualityFlagExplanation ) != OK)
    HALT2(ACT04,"Error getting ScienceQualityFlagExplanation");
  q = km.ScienceQualityFlagExplanation;
  if(PGS_MET_SetAttr(Meta500Handles[1],"ScienceQualityFlagExplanation.1",&q ) != OK )
    HALT2(ACT05,"FAILURE putting ScienceQualityFlagExplanation");

  q = km.DayNightFlag;
  if(PGS_MET_SetAttr(Meta500Handles[1],"DayNightFlag",&q ) != OK )
    HALT2(ACT05,"FAILURE putting DayNightFlag");

  strcpy( km.LocalVersionID, LOCAL_VERSION_ID );
  q = km.LocalVersionID;
  if(PGS_MET_SetAttr(Meta500Handles[1],"LocalVersionID",&q ) != OK )
    HALT2(ACT05,"FAILURE putting LocalVersionID");

  version = 1;
  if( PGS_PC_GetUniversalRef( L1B_INPUT, version_p, URLin ) != OK)
    HALT2(ACT04,"Cannot get URL for 500m L1B file.");

  version = 1;
  if( PGS_PC_GetUniversalRef( L1B1KM_INPUT, version_p, URL1KMin ) != OK)
    HALT2(ACT04,"Cannot get URL for 1km L1B file.");

  version = 1;
  if( PGS_PC_GetUniversalRef( CLOUD_INPUT, version_p, URLcloud ) != OK)
    HALT2(ACT04,"Cannot get URL for CLOUD file.");

  version = 1;
  if( PGS_PC_GetUniversalRef( GEO_INPUT, version_p, URLgeo ) != OK)
    HALT2(ACT04,"Cannot get URL for GEO file.");

  /*** The input files must be for the TERRA satellite. ***/
  if( strncmp( URLin, "MOD", 3) != 0 )
    HALT2(ACT04,"L1B 500m input not MOD for TERRA satellite");
  if( strncmp( URL1KMin, "MOD", 3) != 0 )
    HALT2(ACT04,"L1B 1km input not MOD for TERRA satellite");
  if( strncmp( URLcloud, "MOD", 3) != 0 )
    HALT2(ACT04,"CLOUD input not MOD for TERRA satellite");
  if( strncmp( URLgeo, "MOD", 3) != 0 )
    HALT2(ACT04,"GEO input not MOD for TERRA satellite");

  if(PGS_MET_SetAttr(Meta500Handles[1], "InputPointer", &n ) != OK )
    HALT2(ACT05,"FAILURE putting URLs in InputPointer");
  km.p_InputPointer[0] = n[0];
  km.p_InputPointer[1] = n[1];
  km.p_InputPointer[2] = n[2];
  km.p_InputPointer[3] = n[3];

  strcpy( km.AncillaryInputType, "Geolocation" );
  q = km.AncillaryInputType;
  if(PGS_MET_SetAttr(Meta500Handles[1], "AncillaryInputType.1", &q ) != OK )
    HALT2(ACT05,"FAILURE putting AncillaryInputType");

  /*** AncillaryInputPointer should not have path name. ***/
  strcpy( km.AncillaryInputPointer, 1+strrchr(geo_FileName,'/') );
  q = km.AncillaryInputPointer;
  if(PGS_MET_SetAttr(Meta500Handles[1], "AncillaryInputPointer.1", &q ) != OK )
    HALT2(ACT05,"FAILURE putting L1B in AncillaryInputPointer for GEO file name");

  strcpy( km.ParameterName, "Snow Cover");
  q = km.ParameterName;
  if(PGS_MET_SetAttr(Meta500Handles[1], "ParameterName.1",&q ) != OK )
    HALT2(ACT05,"FAILURE putting ParameterName");


  if( AutomaticQualityFlag == PASSED )
    strcpy( km.AutomaticQualityFlag, "Passed" );
  else
    strcpy( km.AutomaticQualityFlag, "Suspect");
  q = km.AutomaticQualityFlag;
  if(PGS_MET_SetAttr(Meta500Handles[1], "AutomaticQualityFlag.1", &q) != OK)
    HALT2(ACT05,"FAILURE putting AutomaticQualityFlag");

  q = km.AutomaticQualityFlagExplanation;
  if(PGS_MET_SetAttr(Meta500Handles[1], "AutomaticQualityFlagExplanation.1", &q) != OK )
    HALT2(ACT05,"FAILURE putting AutomaticQualityFlagExplanation");

  if( cnt.LandPixels > 0 ) {
    km.QAPercentMissingData = cnt.QA_MissingData*100.0 / cnt.LandPixels + 0.5;
    km.QAPercentCloudCover  = cnt.Snow_CloudObscured * 100.0 / DenomLandDay + 0.5;
  } else {
    km.QAPercentMissingData = 0;
    km.QAPercentCloudCover  = 0;
  }
  if(PGS_MET_SetAttr( Meta500Handles[1], "QAPercentMissingData.1",
		      &km.QAPercentMissingData ) != OK )
    HALT2(ACT05,"FAILURE putting QAPercentMissingData");

  if(PGS_MET_SetAttr(Meta500Handles[1],"QAPercentCloudCover.1",
		     &km.QAPercentCloudCover ) != OK)
    HALT2(ACT05,"FAILURE putting QAPercentCloudCover");


  /*** Do the AdditionalAttributes.n.  The ParamterValue.n must be a char string. ***/

  sprintf(CharString,"%02d", (int8)(pct_nominal+0.5) );
  local = "QAPERCENTGOODQUALITY";
  km.AdditionalAttributeName[0] = local;
  strcpy( km.AdditionalAttributeParm[0], loc2 );
  if(PGS_MET_SetAttr( Meta500Handles[1], "AdditionalAttributeName.1", &local ) != OK )
    HALT2(ACT05,"FAILURE putting AdditionalAttributeName.1");
  if(PGS_MET_SetAttr( Meta500Handles[1],"ParameterValue.1", &loc2 ) != OK )
    HALT2(ACT05,"FAILURE putting ParameterValue.1");


  sprintf(CharString,"%02d", (int8)(pct_abnormal+0.5) );
  local = "QAPERCENTOTHERQUALITY";
  km.AdditionalAttributeName[1] = local;
  strcpy( km.AdditionalAttributeParm[1], loc2 );
  if(PGS_MET_SetAttr( Meta500Handles[1], "AdditionalAttributeName.2", &local ) != OK )
    HALT2(ACT05,"FAILURE putting AdditionalAttributeName.2");
  if(PGS_MET_SetAttr( Meta500Handles[1], "ParameterValue.2", &loc2 ) != OK )
    HALT2(ACT05,"FAILURE putting ParameterValue.2");

  /* This global variable is set by CopyGEOmetaToSnow.c. */
  local = "GRANULENUMBER";
  km.AdditionalAttributeName[2] = local;
  strcpy( km.AdditionalAttributeParm[2], loc3 );
  if(PGS_MET_SetAttr( Meta500Handles[1], "AdditionalAttributeName.3", &local ) != OK )
    HALT2(ACT05,"FAILURE putting AdditionalAttributeName.3");
  if(PGS_MET_SetAttr( Meta500Handles[1], "ParameterValue.3", &loc3 ) != OK )
    HALT2(ACT05,"FAILURE putting ParameterValue.3");


  FloatVal = cnt.Snow_Snow * 100.0 / DenomLandDay;
  sprintf(CharString,"%02.0f", FloatVal);
  local = "SNOWCOVERPERCENT";
  km.AdditionalAttributeName[3] = local;
  strcpy( km.AdditionalAttributeParm[3], loc2 );
  if(PGS_MET_SetAttr( Meta500Handles[1], "AdditionalAttributeName.4", &local ) != OK )
    HALT2(ACT05,"FAILURE putting AdditionalAttributeName.4");
  if(PGS_MET_SetAttr( Meta500Handles[1], "ParameterValue.4", &loc2 ) != OK )
    HALT2(ACT05,"FAILURE putting ParameterValue.4");


  local = "MOD_PR10 V005";
  strcpy( km.AlgorithmPackageName, local );
  if(PGS_MET_SetAttr(Meta500Handles[2],"AlgorithmPackageName",&local ) != OK )
    HALT2(ACT05,"FAILURE putting AlgorithmPackageName");

  strcpy( km.ProcessingCenter, "MODAPS" );
  q = km.ProcessingCenter;
  if(PGS_MET_SetAttr(Meta500Handles[2],"ProcessingCenter",&q ) != OK )
    HALT2(ACT05,"FAILURE putting ProcessingCenter");

  if(PGS_PC_GetConfigData(PROCESSING_ENVIRONMENT, km.ProcessingEnvironment) != OK)
    HALT2(ACT04,"Error getting ProcessingEnvironment from PCF");
  q = km.ProcessingEnvironment;
  if(PGS_MET_SetAttr(Meta500Handles[2],"ProcessingEnvironment",&q ) != OK )
    HALT2(ACT05,"FAILURE putting ProcessingEnvironment");

  strcpy( km.SPSOParameters, "none" );
  q = km.SPSOParameters;
  if(PGS_MET_SetAttr(Meta500Handles[2],"SPSOParameters",&q ) != OK )
    HALT2(ACT05,"FAILURE putting SPSOParameters");

  strcpy( FILE_in, strrchr(in_FileName, '/')+1 );
  strcpy( FILE_in1km, strrchr(in1km_FileName, '/')+1 );
  strcpy( FILE_geo, strrchr(geo_FileName, '/')+1 );
  strcpy( FILE_cloud, strrchr(cloud_FileName, '/')+1 );
  if(PGS_MET_SetAttr( Meta500Handles[2], "LocalInputGranuleID", &f ) != OK )
    HALT2(ACT05,"FAILURE putting LocalInputGranuleID");
  km.p_LocalInputGranuleID[0] = f[0];
  km.p_LocalInputGranuleID[1] = f[1];
  km.p_LocalInputGranuleID[2] = f[2];
  km.p_LocalInputGranuleID[3] = f[3];

  /*** Retrieve AlgorithmPackage parameters from PCF file.  ***/
  if(PGS_PC_GetConfigData(ALGORITHM_PACKAGE_ACCEPTANCE_DATE,
			 km.AlgorithmPackageAcceptanceDate) != OK)
    HALT2(ACT04,"Error getting AlgorithmPackageAcceptanceDate from PCF");
  q = km.AlgorithmPackageAcceptanceDate;
  if(PGS_MET_SetAttr( Meta500Handles[2], "AlgorithmPackageAcceptanceDate", &q ) != OK )
    HALT2(ACT05,"FAILURE putting AlgorithmPackageAcceptanceDate");

  if(PGS_PC_GetConfigData(ALGORITHM_PACKAGE_MATURITY_CODE,
			 km.AlgorithmPackageMaturityCode) != OK)
    HALT2(ACT04,"Error getting AlgorithmPackageMaturityCode from PCF");
  q = km.AlgorithmPackageMaturityCode;
  if(PGS_MET_SetAttr( Meta500Handles[2], "AlgorithmPackageMaturityCode", &q ) != OK )
    HALT2(ACT05,"FAILURE putting AlgorithmPackageMaturityCode");

  if(PGS_PC_GetConfigData(ALGORITHM_PACKAGE_VERSION,
			 km.AlgorithmPackageVersion) != OK)
    HALT2(ACT04,"Error getting AlgorithmPackageVersion from PCF");
  q = km.AlgorithmPackageVersion;
  if(PGS_MET_SetAttr( Meta500Handles[2], "AlgorithmPackageVersion", &q ) != OK )
    HALT2(ACT05,"FAILURE putting AlgorithmPackageVersion");

  if(PGS_PC_GetConfigData(PGE_VERSION, km.PGEversion) != OK)
    HALT2(ACT04,"Error getting PGEversion from PCF");
  q = km.PGEversion;
  if(PGS_MET_SetAttr( Meta500Handles[1], "PGEversion", &q ) != OK )
    HALT2(ACT05,"FAILURE putting PGEversion");

  {
    /*** Construct ProcessingDateTime from UNIX time. ***/
    time_t	clock;
    struct tm	*gm_p;

    clock = time(NULL);
    gm_p = gmtime( &clock );

    sprintf(km.ProcessingDateTime, "%i-%02i-%02iT%02i:%02i:%02i.000Z",
	    gm_p->tm_year+1900, gm_p->tm_mon+1, gm_p->tm_mday,
	    gm_p->tm_hour, gm_p->tm_min, gm_p->tm_sec );

    q = km.ProcessingDateTime;

    if(PGS_MET_SetAttr(Meta500Handles[2], "ProcessingDateTime", &q ) != OK )
      HALT2(ACT05,"FAILURE putting ProcessingDateTime");
  }
}
