/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_Process_5kmSnowFile.c,v 1.2 2005/05/19 18:17:29 powell Exp $

***************************************************************************************
!C

!Description:
	This function is responsible for writing out the 5 km "Longitude", "Latitude",
	"Snow Cover 5km", and "Snow Cover PixelQA 5km".

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	SNOW_5KM_OUTPUT	The PCF number for the snow output file name
	CallType	Operation to be performed (OPEN,WRITE,CLOSE,TERMINATE)

!Output Parameters:
	None

!Revision History:

   $Log: MOD_PR10_Process_5kmSnowFile.c,v $
   Revision 1.2  2005/05/19 18:17:29  powell
   Fix phone number.

   Revision 1.1  2005/05/19 17:30:23  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:

!Design Notes:
	________________________________________________________________________________
!END
***************************************************************************************/

#include <stdio.h>
#include <string.h>
#include "PGS_IO.h"
#include "PGS_MET.h"
#include "PGS_PC.h"
#include "PGS_MEM.h"
#include "hdf.h"
#include "hntdefs.h"
#include "mfhdf.h"
#include "HdfEosDef.h"
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

/*** Global Variables ***/
extern char	HaltMsg[];
extern char	out_FileName[];		/* 500 meter snow output file for input here. */

/*** The column and row dimensions of the L1B earth view input channels. ***/
/*** The present scan (line) number being processed. ***/
extern int32	No5kmSamps, No5kmScans;
extern char	RCSidentString[];

extern struct Meta5km	km;

/*====================================================================================*/

void Process5kmSnowFile( enum OpType CallType )
{
  char *rcs="$Id: MOD_PR10_Process_5kmSnowFile.c,v 1.2 2005/05/19 18:17:29 powell Exp $";
  static int32	SWfidout_5km, SWidSnow_5km;			/* Swath handles */
  static int32	SnowHDFhandle;
  static PGSt_MET_all_handles	Meta5kmHandles;

  /*----------------------------------------------------------------------------------*/

  if( CallType == OPEN ) {

    char		out_5kmFileName[PGSd_PC_FILE_PATH_MAX];	/* for out file name */
    static int32	hdfID;
    PGSt_integer	version;				/* for HDF input file */

    /*** Add the RCS identification to the global string. ***/
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );


    version = 1;
    if(PGS_PC_GetReference(SNOW_5KM_OUTPUT, &version, out_5kmFileName) != OK)
      HALT2(ACT04,"Error getting snow output file name from PCF");


    /*** Get swath handle ***/
    if( (SWfidout_5km = SWopen( out_5kmFileName, DFACC_CREATE )) == FAIL )
      HALT2(ACT08,"SWopen failed for snow output file");

    /*** Link the swath handle to to toolkit. ***/
    if( EHidinfo(SWfidout_5km, &hdfID, &SnowHDFhandle) == FAIL )
      HALT2(ACT10,"EHindinfo failure");

    /*** Create the swath ***/
    if( (SWidSnow_5km=SWcreate( SWfidout_5km, "MOD_Swath_Snow_5km" )) == FAIL )
      HALT2(ACT10,"SWcreate failed for snow cover swath");

    /*** Create swath dimensions ***/
    if( SWdefdim( SWidSnow_5km, "Coarse_swath_lines_5km", No5kmScans ) == FAIL )
      HALT2(ACT10,"SWdefdim failed 'Coarse_swath_lines_5km' for snow cover swath");

    if( SWdefdim( SWidSnow_5km, "Coarse_swath_pixels_5km",No5kmSamps ) == FAIL )
      HALT2(ACT10,"SWdefdim failed 'Coarse_swath_pixels_5km' for snow cover swath");

    if( SWdefgeofield( SWidSnow_5km, "Longitude",
		       "Coarse_swath_lines_5km,Coarse_swath_pixels_5km",
		       DFNT_FLOAT32, HDFE_NOMERGE ) == FAIL )
      HALT2(ACT10,"SWdefgeofield failed for 5km Longitude");

    if( SWdefgeofield(SWidSnow_5km, "Latitude",
		      "Coarse_swath_lines_5km,Coarse_swath_pixels_5km",
		      DFNT_FLOAT32, HDFE_NOMERGE ) == FAIL )
      HALT2(ACT10,"SWdefgeofield failed for 5km Latitude");


    if( SWdefdatafield( SWidSnow_5km, "Snow_Cover_5km",
			"Coarse_swath_lines_5km,Coarse_swath_pixels_5km",
			DFNT_UINT8, HDFE_NOMERGE ) == FAIL )
      HALT2(ACT10,"SWdefdatafield failed Snow_Cover_5km");

    if( SWdefdatafield( SWidSnow_5km, "Snow_Cover_Pixel_QA_5km",
			"Coarse_swath_lines_5km,Coarse_swath_pixels_5km",
			DFNT_UINT8, HDFE_NOMERGE ) == FAIL )
      HALT2(ACT10,"SWdefdatafield failed Snow_Cover_Pixel_QA_5km");

    if( SWdetach( SWidSnow_5km ) == FAIL )
      HALT2(ACT10,"SWdetach failed");

    if( (SWidSnow_5km=SWattach( SWfidout_5km, "MOD_Swath_Snow_5km" )) == FAIL )
      HALT2(ACT10,"SWattach failed to re-attach swath");

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == WRITE ) {
    int32	numBytes, start[] = { 0, 0 }, SWfidin, SWidin;
    int32	start500m[] = { 5,5 },  stride500m[] = { 10,10 }, edge500m[2];
    float32	*pntr;

    /*** Get swath handle ***/
    if( (SWfidin = SWopen( out_FileName, DFACC_READ )) == FAIL )
      HALT2(ACT04,"SWopen failed for 500 meter file");

    if( (SWidin = SWattach( SWfidin, "MOD_Swath_Snow" )) == FAIL )
      HALT2(ACT09,"SWattach failed for 500 meter snow file");


    /*** Allocate temporary space for the latitude/longitude & data arrays. ***/
    numBytes = No5kmScans * No5kmSamps * sizeof(float32);

    pntr = (float32 *)NULL;
    if( PGS_MEM_Malloc( (void **)&pntr, numBytes ) != OK )
      HALT2(ACT10,"Cannot allocate memory for array.");


    if(SWreadfield( SWidin,"Longitude", start, NULL, NULL, pntr) == FAIL)
      HALT2(ACT06,"Failure reading 'Longitude' from the snow file.");

    if( SWwritefield( SWidSnow_5km, "Longitude", start, NULL, NULL, pntr ) == FAIL )
      HALT2(ACT06,"SWwritefield failed for longitude.");


    if(SWreadfield( SWidin,"Latitude", start, NULL, NULL, pntr) == FAIL)
      HALT2(ACT06,"Failure reading 'Latitude' from the snow file.");

    if( SWwritefield( SWidSnow_5km, "Latitude", start, NULL, NULL, pntr ) == FAIL )
      HALT2(ACT06,"SWwritefield failed for 5km latitude.");

    edge500m[0] = No5kmScans;
    edge500m[1] = No5kmSamps;
    if(SWreadfield( SWidin,"Snow_Cover", start500m, stride500m, edge500m, pntr) == FAIL)
      HALT2(ACT06,"Failure reading 'Snow Cover' from the snow file.");

    if( SWwritefield( SWidSnow_5km, "Snow_Cover_5km", 0, NULL, NULL, pntr ) == FAIL )
      HALT2(ACT06,"SWwritefield failed for a scan of 5km SnowCover");


    if(SWreadfield(SWidin,"Snow_Cover_Pixel_QA",start500m,stride500m,edge500m,pntr)==FAIL)
      HALT2(ACT06,"Failure reading 'Snow_Cover_Pixel_QA' from the snow file.");

    if( SWwritefield( SWidSnow_5km,"Snow_Cover_Pixel_QA_5km",0,NULL,NULL,pntr ) == FAIL )
      HALT2(ACT06,"SWwritefield failed for a scan of 5km SnowQA");

    /*** Free the memory that was used for the LatLon & data arrays. ***/
    PGS_MEM_Free( pntr );

    /*** Close the 500 meter snow file after the data sets have been copied to the
	 5 km output file. ***/
    if( SWdetach( SWidin ) == FAIL )
      HALT2(ACT10,"SWdetach failed for 500m snow file.");

    if( SWclose( SWfidin ) == FAIL )
      HALT2(ACT10,"SWclose failed for 500m snow file.");

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == CLOSE ) {

    int32	sds_id, sds_index;
    char	*longname	= "Coarse resolution (5km) snow cover";
    char	*longnameQA	= "Coarse resolution (5km) snow cover spatial QA";
    char	*units		= "none";
    char	*unitsQA	= "none";
    char	*coordsys	= "cartesian";
    uint8	RangeVals[2]	= {0,254},	FillValue = FILL_VALUE;
    uint8	RangeValsQA[2]	= {0,254};

    char	*longnameLat = "Coarse 5 km resolution latitude";
    char	*longnameLon = "Coarse 5 km resolution longitude";
    char	*unitsLatLon = "degrees";
    float32	RangeValsLat[] = { -90., 90. };
    float32	RangeValsLon[] = { -180., 180. };
    float32	FillValueLatLon = -999.0;
    char	*source = "MOD03 geolocation product; data read from center pixel in 5 km box";


    if( SWdetach( SWidSnow_5km ) == FAIL )
      HALT2(ACT10,"SWdetach failed for snow cover 5km swath");


    /*** Set Longitude SDS attributes. ***/
    if( (sds_index = SDnametoindex( SnowHDFhandle, "Longitude" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for longitude in the output file");

    if( (sds_id = SDselect( SnowHDFhandle, sds_index ) ) == FAIL )
      HALT2(ACT09,"Cannot get sds_index for longitude in the output file");

    if(SDsetdatastrs(sds_id, longnameLon, unitsLatLon, NULL, NULL) == FAIL)
      HALT2(ACT09,"SDsetdatastrs failed for longitude in output file.");

    if( SDsetattr(sds_id,"valid_range",DFNT_FLOAT32,2,RangeValsLon) == FAIL )
      HALT2(ACT09,"SDsetattr failure for valid_range for SnowCover");

    if( SDsetattr(sds_id,"_FillValue",DFNT_FLOAT32, 1, &FillValueLatLon) == FAIL )
      HALT2(ACT09,"SDsetattr failure for _FillValue for SnowCover");

    if( SDsetattr(sds_id,"source",DFNT_CHAR8,strlen(source),source ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for source for SnowCover");

    if( SDendaccess( sds_id ) == FAIL )
      HALT2(ACT09,"Cannot SDendaccess for Longitude for the output file.");


    /*** Set Latitude SDS attributes. ***/
    if( (sds_index = SDnametoindex( SnowHDFhandle, "Latitude" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for latitude in the output file");

    if( (sds_id = SDselect( SnowHDFhandle, sds_index ) ) == FAIL )
      HALT2(ACT09,"Cannot get sds_index for latitude in the output file");

    if(SDsetdatastrs(sds_id, longnameLat, unitsLatLon, NULL, NULL) == FAIL)
      HALT2(ACT09,"SDsetdatastrs failed for latitude in output file.");

    if( SDsetattr(sds_id,"valid_range",DFNT_FLOAT32,2,RangeValsLat ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for valid_range for SnowCover");

    if( SDsetattr(sds_id,"_FillValue",DFNT_FLOAT32, 1, &FillValueLatLon) == FAIL )
      HALT2(ACT09,"SDsetattr failure for _FillValue for SnowCover");

    if( SDsetattr(sds_id,"source",DFNT_CHAR8,strlen(source),source ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for source for SnowCover");

    if( SDendaccess( sds_id ) == FAIL )
      HALT2(ACT09,"Cannot SDendaccess for Latitude for the output file.");


    /*** Write out the "Snow_Cover_5km" SDS attributes. ***/
    if( (sds_index = SDnametoindex( SnowHDFhandle, "Snow_Cover_5km" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for Snow_Cover_5km in the output file");

    if( (sds_id = SDselect( SnowHDFhandle, sds_index)) == FAIL )
      HALT2(ACT09,"SDselect failure for Snow_Cover_5km");

    if(SDsetdatastrs(sds_id, longname, units, NULL, coordsys) == FAIL)
      HALT2(ACT09,"SDsetdatastrs failed for Snow_Cover_5km");

    if( SDsetattr(sds_id,"valid_range", DFNT_UINT8, 2, RangeVals ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for valid_range for Snow_Cover_5km");

    if( SDsetattr(sds_id, "_FillValue", DFNT_UINT8, 1, &FillValue ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for _FillValue for Snow_Cover_5km");

    {
      char s[256];
      sprintf(s,"%i=missing data, %i=no decision, %i=night, %i=no snow, %i=lake, %i=ocean, %i=cloud, %i=lake ice, %i=snow, %i=detector saturated, %i=fill",
	      L1B_MISSING_DATA, NO_DECISION, NIGHT, NO_SNOW, LAKE, OCEAN,
	      CLOUD_OBSCURED, LAKE_ICE, SNOW, DETECTOR_SATURATED, FILL_VALUE );

      if( SDsetattr(sds_id, "Key:", DFNT_CHAR8, strlen(s), s ) == FAIL )
	HALT2(ACT09,"SDsetattr failure for Key for Snow_Cover_5km");
    }

    if( SDendaccess( sds_id) == FAIL )
      HALT2(ACT09,"SDendaccess failure for Snow_Cover_5km");


    /*** Write out the "Snow_Cover_Pixel_QA_5km" SDS attributes. ***/
    if( (sds_index = SDnametoindex( SnowHDFhandle, "Snow_Cover_Pixel_QA_5km" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for Snow_Cover_Pixel_QA_5km in the output file");

    if( (sds_id = SDselect( SnowHDFhandle, sds_index)) == FAIL )
      HALT2(ACT09,"SDselect failure for Snow_Cover_Pixel_QA_5km");

    if(SDsetdatastrs(sds_id, longnameQA, unitsQA, NULL, coordsys) == FAIL)
      HALT2(ACT09,"SDsetdatastrs failed for Snow_Cover_Pixel_QA_5km");

    if( SDsetattr(sds_id,"valid_range", DFNT_UINT8, 2, RangeValsQA ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for valid_range for Snow_Cover_Pixel_QA_5km");

    if( SDsetattr(sds_id, "_FillValue", DFNT_UINT8, 1, &FillValue ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for _FillValue for Snow_Cover_Pixel_QA_5km");

    {
      char *s="0=good quality, 1=other quality, 252=Antarctica mask, 253=land mask, 254=ocean mask, 255=fill";
      if( SDsetattr(sds_id, "Key:", DFNT_CHAR8, strlen(s), s ) == FAIL )
	HALT2(ACT09,"SDsetattr failure for Key for Snow_Cover_Pixel_QA_5km");
    }

    if( SDendaccess( sds_id) == FAIL )
      HALT2(ACT09,"SDendaccess failure for Snow_Cover_Pixel_QA_5km");

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == TERMINATE ) {
      void	*local;

  /*** The RCS identification strings have now all been put in the RCSidentString.
       Now write this out as meta data. ***/
    if( SDsetattr( SnowHDFhandle,"SCF Algorithm Version", DFNT_CHAR,
		   strlen(RCSidentString), RCSidentString ) != OK )
      HALT2(ACT02,"Can't SDsetattr for the program's RCS version strings to 5 km file.");

    if(PGS_MET_Init(MCF_FILE_5KM, Meta5kmHandles) != OK)
      HALT2(ACT04,"Failure in PGS_MET_Init for 5 km");


    local = km.LocalGranuleID;
    if(PGS_MET_SetAttr(Meta5kmHandles[1],"LocalGranuleID",&local ) != OK )
      HALT2(ACT05,"FAILURE putting LocalGranuleID in CORE");

    local = km.DayNightFlag;
    if(PGS_MET_SetAttr(Meta5kmHandles[1],"DayNightFlag",&local ) != OK )
      HALT2(ACT05,"FAILURE putting DayNightFlag in CORE");

    local = km.ReprocessingActual;
    if(PGS_MET_SetAttr(Meta5kmHandles[1],"ReprocessingActual",&local ) != OK )
      HALT2(ACT05,"FAILURE putting ReprocessingActual in CORE");

    local = km.ReprocessingPlanned;
    if(PGS_MET_SetAttr(Meta5kmHandles[1],"ReprocessingPlanned",&local ) != OK)
      HALT2(ACT05,"FAILURE putting ReprocessingPlanned in CORE");

    local = km.ScienceQualityFlagExplanation;
    if(PGS_MET_SetAttr(Meta5kmHandles[1],"ScienceQualityFlagExplanation.1",&local) != OK)
      HALT2(ACT05,"FAILURE putting ScienceQualityFlagExplanation in CORE");

    local = km.LocalVersionID;
    if(PGS_MET_SetAttr(Meta5kmHandles[1],"LocalVersionID",&local ) != OK )
      HALT2(ACT05,"FAILURE putting LocalVersionID in CORE");

    local = km.AutomaticQualityFlagExplanation;
    if(PGS_MET_SetAttr(Meta5kmHandles[1],"AutomaticQualityFlagExplanation.1",&local)!=OK)
      HALT2(ACT05,"FAILURE putting AutomaticQualityFlagExplanation in CORE");

    local = km.AutomaticQualityFlag;
    if(PGS_MET_SetAttr( Meta5kmHandles[1], "AutomaticQualityFlag.1", &local ) != OK )
      HALT2(ACT05,"FAILURE putting AutomaticQualityFlag in CORE");

    if(PGS_MET_SetAttr( Meta5kmHandles[1], "QAPercentMissingData.1",
			&km.QAPercentMissingData ) != OK )
      HALT2(ACT05,"FAILURE putting QAPercentMissingData");

    if(PGS_MET_SetAttr(Meta5kmHandles[1],"QAPercentCloudCover.1",
		       &km.QAPercentCloudCover ) != OK )
      HALT2(ACT05,"FAILURE putting QAPercentCloudCover");

    local = km.ParameterName;
    if(PGS_MET_SetAttr(Meta5kmHandles[1],"ParameterName.1",&local ) != OK )
      HALT2(ACT05,"FAILURE putting ParameterName in CORE");

    local = km.EquatorCrossingDate;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "EquatorCrossingDate.1", &local ) != OK )
      HALT2(ACT05,"FAILURE putting EquatorCrossingDate in CORE");

    local = km.EquatorCrossingTime;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "EquatorCrossingTime.1", &local ) != OK )
      HALT2(ACT05,"FAILURE putting EquatorCrossingTime in CORE");

    if(PGS_MET_SetAttr(Meta5kmHandles[1], "OrbitNumber.1", &km.OrbitNumber ) != OK )
      HALT2(ACT05,"FAILURE putting OrbitNumber in CORE");

    if(PGS_MET_SetAttr(Meta5kmHandles[1], "EquatorCrossingLongitude.1",
		       &km.EquatorCrossingLongitude ) != OK )
      HALT2(ACT05,"FAILURE putting EquatorCrossingLongitude in CORE");

    if(PGS_MET_SetAttr(Meta5kmHandles[1], "InputPointer", &km.p_InputPointer ) != OK )
      HALT2(ACT05,"FAILURE putting URLs in InputPointer");

    if(PGS_MET_SetAttr(Meta5kmHandles[1],
		       "GRingPointLatitude.1", &km.GRingPointLatitude ) != OK )
      HALT2(ACT05,"FAILURE putting GRingPointLatitude");

    if(PGS_MET_SetAttr(Meta5kmHandles[1],
		       "GRingPointLongitude.1", &km.GRingPointLongitude ) != OK )
      HALT2(ACT05,"FAILURE putting GRingPointLongitude");

    if(PGS_MET_SetAttr(Meta5kmHandles[1],
		       "GRingPointSequenceNo.1", &km.GRingPointSequenceNo ) != OK )
      HALT2(ACT05,"FAILURE putting GRingPointSequenceNo");

    local = km.ExclusionGRingFlag;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "ExclusionGRingFlag.1", &local ) != OK )
      HALT2(ACT05,"FAILURE putting ExclusionGRingFlag");

    local = &km.RangeBeginningDate;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "RangeBeginningDate", &local ) != OK )
      HALT2(ACT05,"FAILURE putting RangeBeginningDate");

    local = &km.RangeEndingDate;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "RangeEndingDate", &local ) != OK )
      HALT2(ACT05,"FAILURE putting RangeEndingDate");

    local = &km.RangeBeginningTime;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "RangeBeginningTime", &local ) != OK )
      HALT2(ACT05,"FAILURE putting RangeBeginningTime");

    local = &km.RangeEndingTime;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "RangeEndingTime", &local ) != OK )
      HALT2(ACT05,"FAILURE putting RangeEndingTime");

    local = &km.PGEversion;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "PGEversion", &local ) != OK )
      HALT2(ACT05,"FAILURE putting PGEversion");

    local = &km.AncillaryInputType;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "AncillaryInputType.1", &local ) != OK )
      HALT2(ACT05,"FAILURE putting AncillaryInputType");

    local = &km.AncillaryInputPointer;
    if(PGS_MET_SetAttr(Meta5kmHandles[1], "AncillaryInputPointer.1", &local ) != OK )
      HALT2(ACT05,"FAILURE putting AncillaryInputPointer");

    {
      char	name[42], parm[64];
      void	*p[2], *loc2;
      int16	j;
      p[0] = name;
      p[1] = parm;
      for( j=0; j<ADDITIONAL_ATTRIBUTES; j++ ) {
	sprintf(name,"AdditionalAttributeName.%d", j+1 );
	sprintf(parm,"ParameterValue.%d", j+1);
	local = km.AdditionalAttributeName[j];
	loc2  = km.AdditionalAttributeParm[j];
	if(PGS_MET_SetAttr( Meta5kmHandles[1], p[0], &local ) != OK )
	  HALT2(ACT05,"FAILURE putting AdditionalAttributeName");
	if(PGS_MET_SetAttr( Meta5kmHandles[1], p[1], &loc2  ) != OK )
	  HALT2(ACT05,"FAILURE putting ParameterValue");
      }
    }

    local = km.AlgorithmPackageAcceptanceDate;
    if(PGS_MET_SetAttr(Meta5kmHandles[2],"AlgorithmPackageAcceptanceDate",&local) != OK )
      HALT2(ACT05,"FAILURE putting AlgorithmPackageAcceptanceDate");

    local = km.AlgorithmPackageMaturityCode;
    if(PGS_MET_SetAttr( Meta5kmHandles[2],"AlgorithmPackageMaturityCode",&local ) != OK )
      HALT2(ACT05,"FAILURE putting AlgorithmPackageMaturityCode");

    local = km.AlgorithmPackageVersion;
    if(PGS_MET_SetAttr( Meta5kmHandles[2], "AlgorithmPackageVersion", &local ) != OK )
      HALT2(ACT05,"FAILURE putting AlgorithmPackageVersion");

    local = km.AlgorithmPackageName;
    if(PGS_MET_SetAttr( Meta5kmHandles[2], "AlgorithmPackageName", &local ) != OK )
      HALT2(ACT05,"FAILURE putting AlgorithmPackageName");

    local = km.ProcessingCenter;
    if(PGS_MET_SetAttr( Meta5kmHandles[2],"ProcessingCenter",&local ) != OK )
      HALT2(ACT05,"FAILURE putting ProcessingCenter");

    local = km.ProcessingEnvironment;
    if(PGS_MET_SetAttr( Meta5kmHandles[2],"ProcessingEnvironment",&local ) != OK )
      HALT2(ACT05,"FAILURE putting ProcessingEnvironment");

    local = km.ProcessingDateTime;
    if(PGS_MET_SetAttr( Meta5kmHandles[2],"ProcessingDateTime",&local ) != OK )
      HALT2(ACT05,"FAILURE putting ProcessingDateTime");

    local = km.SPSOParameters;
    if(PGS_MET_SetAttr( Meta5kmHandles[2],"SPSOParameters",&local ) != OK )
      HALT2(ACT05,"FAILURE putting SPSOParameters");

    if(PGS_MET_SetAttr( Meta5kmHandles[2],"LocalInputGranuleID"
			,&km.p_LocalInputGranuleID ) != OK )
      HALT2(ACT05,"FAILURE putting LocalInputGranuleID");



    if(PGS_MET_SetAttr(Meta5kmHandles[2],
		       "EastBoundingCoordinate", &km.EastBoundingCoordinate) != OK )
      HALT2(ACT05,"FAILURE putting EastBoundingCoordinate");

    if(PGS_MET_SetAttr(Meta5kmHandles[2],
		       "WestBoundingCoordinate", &km.WestBoundingCoordinate) != OK )
      HALT2(ACT05,"FAILURE putting WestBoundingCoordinate");

    if(PGS_MET_SetAttr(Meta5kmHandles[2],
		       "NorthBoundingCoordinate", &km.NorthBoundingCoordinate) != OK )
      HALT2(ACT05,"FAILURE putting NorthBoundingCoordinate");

    if(PGS_MET_SetAttr(Meta5kmHandles[2],
		       "SouthBoundingCoordinate", &km.SouthBoundingCoordinate) != OK )
      HALT2(ACT05,"FAILURE putting SouthBoundingCoordinate");


    /*** Write out the meta data to the snow output file. ***/
    if(PGS_MET_Write(Meta5kmHandles[1], CORE, SnowHDFhandle) != OK)
      HALT2(ACT05,"PGS_MET_Write -- Error writing CORE 5 km");

    if(PGS_MET_Write(Meta5kmHandles[2], ARCHIVE, SnowHDFhandle) != OK)
      HALT2(ACT05,"PGS_MET_Write -- Error writing ARCHIVE 5 km");

    PGS_MET_Remove();


    if( SWclose( SWfidout_5km ) == FAIL )
      HALT2(ACT10,"SWclose failed for output file");
  }

}
