/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_UpdateCounts.c,v 1.1 2005/05/19 17:30:23 powell Exp $

****************************************************************************************
!C

!Description:
	This function is responsible for keeping counts of the various snow and
	QA values that are written for the 500 meter product.

Program returns:
	Always returns successfully.  There no error messages.

!Input Parameters:
	snow_p		Snow cover scan pointer
	snowQA_p	Snow QA scan pointer

!Revision History:
	$Log: MOD_PR10_UpdateCounts.c,v $
	Revision 1.1  2005/05/19 17:30:23  powell
	Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:

!Design Notes:
	This is the Collection 5 version of the code.  This routine is called once after
	each line has been written by MOD_PR10_Process_SnowFile.c.

!END
***************************************************************************************/

#include <stdio.h>
#include <string.h>
#include "hdf.h"
#include "MOD_PR10.h"

extern int32		NoSamps;
extern struct COUNTS	cnt;
extern char		RCSidentString[];

/*====================================================================================*/

void UpdateCounts( uint8 * sno_p, uint8 * qa_p )
{
  char *rcs="$Id: MOD_PR10_UpdateCounts.c,v 1.1 2005/05/19 17:30:23 powell Exp $";
  int	j;
  static char	FirstTimeFlag = XTRUEX;

  /*** Add the RCS identification to the global string. ***/
  if( FirstTimeFlag == XTRUEX ) {
    FirstTimeFlag = XFALSEX;
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );
  }

  for( j=0; j<NoSamps; j++ ) {

    switch (*(sno_p+j) ) {
    case FILL_VALUE:		cnt.Snow_FillValue		++;	break;
    case DETECTOR_SATURATED:	cnt.Snow_DetectorSaturated	++;	break;
    case SNOW:			cnt.Snow_Snow			++;	break;
    case CLOUD_OBSCURED:	cnt.Snow_CloudObscured		++;	break;
    case OCEAN:			cnt.Snow_Ocean			++;	break;
    case LAKE:			cnt.Snow_Lake			++;	break;
    case LAKE_ICE:		cnt.Snow_LakeIce		++;	break;
    case NO_SNOW:	        cnt.Snow_NoSnow			++;	break;
    case NIGHT:			cnt.Snow_Night			++;	break;
    case NO_DECISION:		cnt.Snow_NoDecision		++;	break;
    case L1B_MISSING_DATA:	cnt.Snow_L1BMissingData		++;
    }

    switch( *(qa_p+j) ) {
    case QA_GOOD:		cnt.QA_Good ++;		break;
    case QA_OCEAN:		cnt.QA_Ocean ++;	break;
    }
  }
  return;
}

